# -*- coding: utf-8 -*-
from django.test import TestCase, TransactionTestCase
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.core.files.base import File
from django.http import HttpRequest, QueryDict
from django.utils.encoding import smart_str

from subprocess import Popen, PIPE
from datetime import date
from M2Crypto import m2, ASN1, RSA, EVP, X509, BIO, SMIME
from datetime import datetime
from tempfile import NamedTemporaryFile, TemporaryFile

from signature.models import Key, Certificate, CertificateRequest, Signature
from signature.openssl import Openssl
from certificates import C_KEY, CA_KEY ,C_PUB_KEY, CA_CERT, C_REQUEST, C_CERT, U_CERT, U_KEY, U_REQUEST, UTF8_CERT
from certificates import C_KEY_PATH, CA_KEY_PATH, C_PUB_KEY_PATH, CA_CERT_PATH, C_REQUEST_PATH, C_CERT_PATH, U_CERT_PATH, U_KEY_PATH, U_REQUEST_PATH, UTF8_CERT_PATH

from models import Author, Whatamess, Book

import os
import shlex

class SignaturePKITestCase(TestCase):
    """Tests with django Signature + M2Cryto
    """
    def testKeyGenerationPrivate(self):
        """Test Private Key pair generation
        """
        user_pwd = "tata"
        k = Key.generate(user_pwd)
        k.save()
        self.assertTrue("-----BEGIN RSA PRIVATE KEY-----" in k.private)
        self.assertTrue("ENCRYPTED" in k.private)
        self.assertTrue("-----BEGIN PUBLIC KEY-----" in k.public)
        pkey = k.m2_pkey(user_pwd)
        self.assertTrue(isinstance(pkey, EVP.PKey))

    def testKeyGenerationPrivateUtf8(self):
        """Test Key pair generation with utf8 encoded password
        """
        password = "ééééèèèèçççùùù"
        k = Key.generate(password)
        k.save()
        k = Key.objects.get(id=k.id)
        self.assertTrue("-----BEGIN RSA PRIVATE KEY-----" in k.private)
        self.assertTrue("ENCRYPTED" in k.private)
        self.assertTrue("-----BEGIN PUBLIC KEY-----" in k.public)
        pkey = k.m2_pkey(password)
        self.assertTrue(isinstance(pkey, EVP.PKey))

    def testKeyGeneration(self):
        """Test Key pair generation without encryption
        """
        k = Key.generate(None)
        k.save()
        self.assertTrue("-----BEGIN RSA PRIVATE KEY-----" in k.private)
        self.assertTrue("ENCRYPTED" not in k.private)
        self.assertTrue("-----BEGIN PUBLIC KEY-----" in k.public)
        pkey = k.m2_pkey()
        self.assertTrue(isinstance(pkey, EVP.PKey))

    def testKeyLoading(self):
        """Try to load key
        """
        k = Key.new_from_pem(C_KEY, "1234")
        self.assertEqual(k.length, 4096)
        self.assertEqual(k.public, C_PUB_KEY)

    def testSelfCertificateGeneration(self):
        """With a Key, try to generate a self-signed certificate
        """
        before = datetime(2010, 01, 01)
        after = datetime(2015, 01, 01)
        user_pwd = "tata"
        key = Key.generate(user_pwd)
        key.save()
        cert = Certificate()
        cert.CN = "My CN"
        cert.country = "FR"
        cert.key = key
        cert.days = 300
        cert.is_ca = True
        cert.generate_x509_root(user_pwd)
        cert.save()
        cert_pem = cert.pem
        #self.assertEqual(cert.serial, 0)
        self.assertEqual(cert.ca_serial, 1)
        self.assertTrue(cert.is_ca)
        self.assertTrue(cert.trust)

        # Just test Certificate.m2_x509() method
        x509 = X509.load_cert_string(cert_pem, X509.FORMAT_PEM)
        m2x509 = cert.m2_x509()
        self.assertTrue(x509.as_text() == m2x509.as_text())

        self.assertTrue("CA:TRUE" in m2x509.as_text())
        self.assertTrue("Issuer: CN=My CN, C=FR" in m2x509.as_text())
        self.assertTrue("Subject: CN=My CN, C=FR" in m2x509.as_text())
        self.assertTrue("X509v3 Authority Key Identifier" in m2x509.as_text())
        self.assertTrue("X509v3 Subject Key Identifier" in m2x509.as_text())
        return cert_pem

    def testCertificateLoading(self):
        """Load x509 certificate
        """
        #before = datetime(2010, 01, 01, 6, tzinfo=ASN1.UTC)
        #after = datetime(2015, 01, 01, 6, tzinfo=ASN1.UTC)
        x509_text = X509.load_cert_string(CA_CERT, X509.FORMAT_PEM).as_text()

        cert = Certificate.new_from_pem(CA_CERT)
        cert.save()
        self.assertTrue(cert.CN == "Admin")
        self.assertTrue(cert.country == "FR")
        #self.assertTrue(cert.begin == before)
        #self.assertTrue(cert.end == after)
        self.assertTrue(cert.is_ca)
        self.assertTrue(cert.auth_kid)
        self.assertTrue(cert.subject_kid)
        self.assertTrue(cert.certhash)
        self.assertTrue(" " not in cert.auth_kid)
        self.assertTrue(" " not in cert.subject_kid)
        # Just test Certificate.m2_x509() method
        x509 = X509.load_cert_string(cert.pem, X509.FORMAT_PEM)
        m2x509 = cert.m2_x509()
        self.assertTrue(x509.as_text() == m2x509.as_text())

        self.assertTrue(cert.auth_kid in m2x509.as_text())
        self.assertTrue(cert.subject_kid in m2x509.as_text())

    def testCertificateLoadingUTF8(self):
        """Load x509 certificate UTF8
        """
        #before = datetime(2010, 01, 01, 6, tzinfo=ASN1.UTC)
        #after = datetime(2015, 01, 01, 6, tzinfo=ASN1.UTC)
        cert = Certificate.new_from_pem(UTF8_CERT)
        cert.save()
        self.assertEqual(cert.CN, u"Admin ©")
        self.assertEqual(cert.country, u"FR")
        cert = Certificate.objects.get(id=cert.id)
        self.assertEqual(cert.CN, u"Admin ©")
        self.assertEqual(cert.country, u"FR")

    def testRequestGeneration(self):
        """With a Key, try to generate a request
        """
        user_pwd = "tata"
        key = Key.generate(user_pwd)
        key.save()
        rqst = CertificateRequest()
        rqst.CN = "World Company"
        rqst.country = "FR"
        rqst.key = key
        rqst.sign_request(user_pwd)
        rqst.save()
        rqst_pem = rqst.pem

        m2rqst = rqst.m2_request()
        self.assertTrue("Subject: CN=World Company, C=FR" in m2rqst.as_text())
        return rqst_pem

    def testRequestLoading(self):
        """Load Request loading
        """
        m2rqst_text = X509.load_request_string(C_REQUEST, X509.FORMAT_PEM).as_text()

        rqst = CertificateRequest.new_from_pem(C_REQUEST)
        rqst.save()
        self.assertTrue(rqst.CN == "World Company")
        self.assertTrue(rqst.country == "FR")
        rqst_text = X509.load_request_string(rqst.pem, X509.FORMAT_PEM).as_text()
        self.assertTrue(rqst_text == m2rqst_text)

    def testSignaturePKI(self):
        """
        Symbol © is for testing utf8
        """
        before = datetime(2010, 01, 01, 6, tzinfo=ASN1.UTC)
        after = datetime(2015, 01, 01, 6, tzinfo=ASN1.UTC)
        ca_pwd = "R00tz"
        c_pwd = "1234"

        # CA and Client keys
        ca_key = Key.generate(ca_pwd)
        #print "\nCA PRIVATE\n", ca_key.private, "\n"
        c_key = Key.generate(c_pwd)
        #print "\nC PRIVATE\n", c_key.private, "\n"
        #print "\nC PUB\n", c_key.public, "\n"

        # CA Cert
        ca_cert = Certificate()
        ca_cert.CN = "Admin ©"
        ca_cert.country = "FR"
        ca_cert.key = ca_key
        ca_cert.begin = before
        ca_cert.end = after
        ca_cert.is_ca = True
        ca_cert.generate_x509_root(ca_pwd)
        ca_cert.save()
        self.assertEqual(ca_cert.ca_serial, 1)
        #print "\nCA cert\n", ca_cert.pem, "\n"

        # Client's request
        rqst = CertificateRequest()
        rqst.CN = "World Company ©"
        rqst.country = "FR"
        rqst.key = c_key
        rqst.sign_request(c_pwd)
        rqst.save()
        #print "\nRQST\n", rqst.pem, "\n"

        c_cert = ca_cert.sign_request(rqst, 300, ca_pwd)
        c_cert.save()
        #print "\nC_CERT\n", c_cert.pem, "\n"
        self.assertEqual(c_cert.serial, '2')
        self.assertEqual(ca_cert.ca_serial, 2)
        self.assertTrue("Signature ok" not in c_cert.pem)
        self.assertFalse(c_cert.trust)
        self.assertTrue(ca_cert.trust)
        self.assertTrue(ca_cert.certhash)
        self.assertTrue(c_cert.certhash)

        c_cert = Certificate.objects.get(id=c_cert.id)
        x509 = X509.load_cert_string(smart_str(c_cert.pem), X509.FORMAT_PEM)
        m2x509 = c_cert.m2_x509()
        self.assertTrue(x509.as_text() == m2x509.as_text())

        self.assertTrue("Issuer: CN=Admin \\xC2\\xA9, C=FR" in m2x509.as_text())
        self.assertTrue("Subject: CN=World Company \\xC2\\xA9, C=FR" in m2x509.as_text())
        self.assertTrue("X509v3 Authority Key Identifier" in m2x509.as_text())
        self.assertTrue("X509v3 Subject Key Identifier" in m2x509.as_text())

    def testSignaturePKIRevoke(self):
        """Try create - revoke - renew
        """
        before = datetime(2010, 01, 01, 6, tzinfo=ASN1.UTC)
        after = datetime(2015, 01, 01, 6, tzinfo=ASN1.UTC)
        ca_pwd = "R00tz"
        c_pwd = "1234"

        # CA and Client keys
        ca_key = Key.generate(ca_pwd)
        ca_key.save()
        #print "\nCA PRIVATE\n", ca_key.private, "\n"
        c_key = Key.generate(c_pwd)
        c_key.save()
        cc_key = Key.generate(c_pwd)
        cc_key.save()
        #print "\nC PRIVATE\n", c_key.private, "\n"
        #print "\nC PUB\n", c_key.public, "\n"

        # CA Cert
        ca_cert = Certificate()
        ca_cert.CN = "Admin"
        ca_cert.country = "FR"
        ca_cert.key = ca_key
        ca_cert.begin = before
        ca_cert.end = after
        ca_cert.is_ca = True
        ca_cert.generate_x509_root(ca_pwd)
        ca_cert.save()
        #print "\nCA cert\n", ca_cert.pem, "\n"

        # Client's request
        rqst = CertificateRequest()
        rqst.CN = "World Company ©"
        rqst.country = "FR"
        rqst.locality = "World"
        rqst.organization = "Company"
        rqst.OU = "Unknown"
        rqst.state = "Dummy"
        rqst.country = "FR"
        rqst.email = "dummy@dummy.fr"
        rqst.key = c_key
        rqst.sign_request(c_pwd)
        rqst.save()
        #print "\nRQST\n", rqst.pem, "\n"

        c_cert = ca_cert.sign_request(rqst, 300, ca_pwd)
        c_cert.save()
        #print "\nC_CERT\n", c_cert.pem, "\n"
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)
        ca_cert.revoke(c_cert, ca_pwd)
        ca_cert.save()
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)

        rqst.delete()
        # Client's new request
        rqst = CertificateRequest()
        rqst.CN = "World Company ©"
        rqst.country = "FR"
        rqst.locality = "World"
        rqst.organization = "Company"
        rqst.OU = "Unknown"
        rqst.state = "Dummy"
        rqst.country = "FR"
        rqst.email = "dummy@dummy.fr"
        rqst.key = c_key
        rqst.sign_request(c_pwd)
        rqst.save()
        #print "\nRQST\n", rqst.pem, "\n"
        c2_cert = ca_cert.sign_request(rqst, 300, ca_pwd)
        c2_cert.save()
        # Revoke new
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c2_cert = Certificate.objects.get(pk=c2_cert.id)
        ca_cert.revoke(c2_cert, ca_pwd)
        ca_cert.save()
        self.assertFalse(c2_cert.check())
        #print ca_cert.index
        #print [ca_cert.index]
        #print ca_cert.crl

        # Try another client
        rqst2 = CertificateRequest()
        rqst2.CN = "Country Company ©"
        rqst2.country = "FR"
        rqst2.locality = "Country"
        rqst2.organization = "Company"
        rqst2.OU = "Unknown"
        rqst2.state = "Dummy"
        rqst2.country = "FR"
        rqst2.email = "dummy2@dummy2.fr"
        rqst2.key = c_key
        rqst2.sign_request(c_pwd)
        rqst2.save()
        #print "\nRQST\n", rqst.pem, "\n"
        cc_cert = ca_cert.sign_request(rqst2, 300, ca_pwd)
        cc_cert.save()
        self.assertTrue(cc_cert.check())
        # Revoke new
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        cc_cert = Certificate.objects.get(pk=cc_cert.id)
        ca_cert.revoke(cc_cert, ca_pwd)
        ca_cert.save()
        self.assertFalse(cc_cert.check())
        #print ca_cert.index
        #print [ca_cert.index]
        #print ca_cert.crl


    def testSignaturePKIca(self):
        """Client certificate is a CA
        """
        # Turn this to True to regenerate examples certificates
        save = False

        before = datetime(2010, 01, 01, 6, tzinfo=ASN1.UTC)
        after = datetime(2015, 01, 01, 6, tzinfo=ASN1.UTC)
        ca_pwd = "R00tz"
        c_pwd = "1234"
        #c2_pwd = "abcd"

        # CA and Client keys
        ca_key = Key.generate(ca_pwd)
        c_key = Key.generate(c_pwd)
        c2_key = Key.generate(None)

        # CA Cert
        ca_cert = Certificate()
        ca_cert.CN = "Admin"
        ca_cert.country = "FR"
        ca_cert.key = ca_key
        ca_cert.days = 3000
        ca_cert.is_ca = True
        ca_cert.generate_x509_root(ca_pwd)

        # Client's request
        rqst = CertificateRequest()
        rqst.CN = "World Company"
        rqst.country = "FR"
        rqst.key = c_key
        rqst.sign_request(c_pwd)

        c_cert = ca_cert.sign_request(rqst, 200, ca_pwd, ca=True)
        self.assertEqual(c_cert.serial, '2')
        self.assertEqual(ca_cert.ca_serial, 2)

        # Just test Certificate.m2_x509() method
        x509 = X509.load_cert_string(c_cert.pem, X509.FORMAT_PEM)
        m2x509 = c_cert.m2_x509()
        self.assertTrue(x509.as_text() == m2x509.as_text())

        self.assertTrue("CA:TRUE" in m2x509.as_text())
        self.assertTrue("Issuer: CN=Admin, C=FR" in m2x509.as_text())
        self.assertTrue("Subject: CN=World Company, C=FR" in m2x509.as_text())
        self.assertTrue("X509v3 Authority Key Identifier" in m2x509.as_text())
        self.assertTrue("X509v3 Subject Key Identifier" in m2x509.as_text())
        self.assertTrue(c_cert.auth_kid)
        self.assertTrue(c_cert.subject_kid)
        self.assertTrue(" " not in c_cert.auth_kid)
        self.assertTrue(" " not in c_cert.subject_kid)
        self.assertTrue(c_cert.auth_kid in m2x509.as_text())
        self.assertTrue(c_cert.subject_kid in m2x509.as_text())
        # get authkey


        # Client's request
        urqst = CertificateRequest()
        urqst.CN = "Country Company"
        urqst.country = "FR"
        urqst.key = c2_key
        urqst.sign_request()

        c2_cert = c_cert.sign_request(urqst, 150, c_pwd)
        self.assertEqual(c2_cert.serial, '2')
        self.assertEqual(c_cert.ca_serial, 2)

        # Just test Certificate.m2_x509() method
        x509 = X509.load_cert_string(c2_cert.pem, X509.FORMAT_PEM)
        m2x509 = c2_cert.m2_x509()
        self.assertTrue(x509.as_text() == m2x509.as_text())

        self.assertTrue("Issuer: CN=World Company, C=FR" in m2x509.as_text())
        self.assertTrue("Subject: CN=Country Company, C=FR" in m2x509.as_text())
        self.assertTrue("X509v3 Authority Key Identifier" in m2x509.as_text())
        self.assertTrue("X509v3 Subject Key Identifier" in m2x509.as_text())
        self.assertTrue(c2_cert.auth_kid)
        self.assertTrue(c2_cert.subject_kid)
        self.assertTrue(c2_cert.auth_kid in m2x509.as_text())
        self.assertTrue(c2_cert.subject_kid in m2x509.as_text())
        self.assertTrue(" " not in c2_cert.auth_kid)
        self.assertTrue(" " not in c2_cert.subject_kid)

        # If all tests are goods, lets save it if needed
        if save:
            # UTF8 CA Cert
            utf8_key = Key.generate(ca_pwd)
            utf8_cert = Certificate()
            utf8_cert.CN = "Admin ©"
            utf8_cert.country = "FR"
            utf8_cert.key = utf8_key
            utf8_cert.begin = before
            utf8_cert.end = after
            utf8_cert.is_ca = True
            utf8_cert.generate_x509_root(ca_pwd)
            utf8_cert.save()
            open(CA_KEY_PATH, 'w').write(ca_key.private)
            open(C_KEY_PATH, 'w').write(c_key.private)
            open(U_KEY_PATH, 'w').write(c2_key.private)
            open(C_PUB_KEY_PATH, 'w').write(c_key.public)
            open(CA_CERT_PATH, 'w').write(ca_cert.pem)
            open(C_REQUEST_PATH, 'w').write(rqst.pem)
            open(C_CERT_PATH, 'w').write(c_cert.pem)
            open(U_CERT_PATH, 'w').write(c2_cert.pem)
            open(UTF8_CERT_PATH, 'w').write(utf8_cert.pem)
            print "SAVED"

    def testCertificateChainLoading(self):
        """Load many x509 and check relations
        """
        user = User(email="f@f.fr", username='toto')
        user.save()
        # Check relations for certs imports
        ca_key = Key.new_from_pem(CA_KEY, "R00tz")
        ca_key.user = user
        ca_key.save()
        ca_cert = Certificate.new_from_pem(CA_CERT)
        ca_cert.save()
        self.assertEqual(ca_cert.key, ca_key)

        # Check relations for keys imports
        c_cert = Certificate.new_from_pem(C_CERT)
        c_cert.save()
        c_key = Key.new_from_pem(C_KEY, "1234")
        # Refresh object
        c_cert = Certificate.objects.get(pk=c_cert.id)
        self.assertEqual(c_cert.key, c_key)

        # Check issuer relations
        u_cert = Certificate.new_from_pem(U_CERT)
        u_cert.save()
        u_key = Key.new_from_pem(U_KEY)
        self.assertTrue(u_cert.issuer == c_cert)
        self.assertTrue(u_cert.issuer.issuer == ca_cert)

    def testCertificateChainLoadingIssued(self):
        """Load many x509 and check _issued_ relations
        """
        user = User(email="f@f.fr", username='toto')
        user.save()
        # Check relations for certs imports
        ca_cert = Certificate.new_from_pem(CA_CERT)
        ca_cert.save()

        u_cert = Certificate.new_from_pem(U_CERT)
        u_cert.save()

        # Check relations for keys imports
        c_cert = Certificate.new_from_pem(C_CERT)
        c_cert.save()
        # Refresh object
        c_cert = Certificate.objects.get(pk=c_cert.id)
        u_cert = Certificate.objects.get(pk=u_cert.id)

        self.assertTrue(c_cert.issuer == ca_cert)
        self.assertTrue(u_cert.issuer == c_cert)

    def testCertificateCheck(self):
        """Load many x509 and check certificates
        """
        ca_pwd = "R00tz"
        c_pwd = "1234"
        # Check relations for certs imports
        ca_cert = Certificate.new_from_pem(CA_CERT)
        ca_cert.save()

        # Check relations for keys imports
        c_cert = Certificate.new_from_pem(C_CERT)
        c_cert.save()
        # Refresh object
        c_cert = Certificate.objects.get(pk=c_cert.id)

        # Check issuer relations
        u_cert = Certificate.new_from_pem(U_CERT)
        u_cert.save()

        self.assertEqual(c_cert.get_cert_chain(), [ca_cert, c_cert])
        self.assertEqual(u_cert.get_cert_chain(), [ca_cert, c_cert, u_cert])
        self.assertRaises(Openssl.VerifyError, ca_cert.check)
        self.assertRaises(Openssl.VerifyError, c_cert.check)
        self.assertRaises(Openssl.VerifyError, u_cert.check)
        ca_cert.trust = True
        ca_cert.save()

        # WTF ? we have to reload all objects after change ca_trust or
        # x_cert.get_cert_chain()[0].trust will be false
        # Tested with TransactionTestCase
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertEqual(c_cert.get_cert_chain()[0].trust, True)

        self.assertTrue(ca_cert.check(crlcheck=False))
        self.assertTrue(c_cert.check(crlcheck=False))
        self.assertTrue(u_cert.check(crlcheck=False))

        # Add crl
        # Use Quick method
        c_cert.revoked = True
        c_cert.save()
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertFalse(u_cert.check(quick=True))
        c_cert.revoked = False
        c_cert.save()
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertTrue(u_cert.check())
        # Use openssl method
        c_cert.crl = "Wrong crl"
        c_cert.save()
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertRaises(Openssl.VerifyError, u_cert.check)
        # TODO : Add real CRL

        # Gen CRL for CA
        k = Key.new_from_pem(CA_KEY, ca_pwd)
        k.save()
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        ca_cert.ca_serial = 2
        ca_cert.save()
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        ca_cert.gen_crl(ca_pwd)
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        self.assertTrue("CRL" in ca_cert.crl)
        # Must works with this CRL
        c_cert.crl = None
        c_cert.save()
        u_cert = Certificate.objects.get(pk=u_cert.id)
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        self.assertTrue(u_cert.check())

        # Revoke client's certificate
        # Try with no crl
        ca_cert.crl = None
        ca_cert.save()
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        ca_cert.revoke(c_cert, ca_pwd)
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertFalse(u_cert.check())
        c_cert.revoked = False
        c_cert.save()
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertFalse(u_cert.check())
        self.assertTrue("02" in ca_cert.index)
        self.assertTrue("World Company" in ca_cert.index)

        # Revocation must be present on other crls
        ca_cert.gen_crl(ca_pwd)
        ca_cert = Certificate.objects.get(pk=ca_cert.id)
        c_cert = Certificate.objects.get(pk=c_cert.id)
        u_cert = Certificate.objects.get(pk=u_cert.id)
        self.assertFalse(u_cert.check())


class SignatureTestCase(TestCase):
    """Tests with django Signature + M2Cryto
    Sign some models
    """
    def setUp(self):
        """Load keys
        """
        self.ca_pwd = "R00tz"
        self.c_pwd = "1234"
        self.user_admin = User.objects.create(username="Admin", email="admin@server.bofh")
        self.user_client = User.objects.create(username="Client", email="client@internet.isp")
        ca_key = Key.new_from_pem(CA_KEY, "R00tz", self.user_admin)
        ca_key.save()
        c_key = Key.new_from_pem(C_KEY, "1234", self.user_client)
        c_key.save()
        ca_cert = Certificate.new_from_pem(CA_CERT, user=self.user_admin, key=ca_key)
        ca_cert.save()
        c_cert = Certificate.new_from_pem(C_CERT, user=self.user_client, key=c_key)
        c_cert.save()
        self.ca_key = Key.objects.get(id=ca_key.id)
        self.c_key = Key.objects.get(id=c_key.id)
        self.ca_cert = Certificate.objects.get(id=ca_cert.id)
        self.c_cert = Certificate.objects.get(id=c_cert.id)


    def testBasicTextPKCS7(self):
        """Try to sign a basic text
        """
        # Sign
        text = "This is a data"
        data_signed = self.c_cert.sign_text(text, self.c_pwd)
        result = self.c_cert.verify_smime(data_signed)
        self.assertTrue(result)
        self.assertEqual(result, text)

    def testBasicModelPKCS7(self):
        """Try to sign a basic model
        """
        # Sign
        auth1 = Author(name="Raymond E. Feist", title="MR")
        auth1.save()
        data_signed = self.c_cert.sign_model(auth1, self.c_pwd)
        result = self.c_cert.verify_smime(data_signed)
        self.assertTrue(result)

    def testComplexModelPKCS7(self):
        """Try to sign a complex model
        """
        # Sign
        auth1 = Author(name="Raymond E. Feist", title="MR")
        auth1.save()
        wam1 = Whatamess()
        wam1.name = "Woaw"
        wam1.number = 1
        wam1.slug = "woot"
        wam1.text= "This is a data"
        wam1.author = auth1
        wam1.title = 1
        wam1.birth_date = datetime.now()
        wam1.yesno = True
        wam1.save()
        data_signed = self.c_cert.sign_model(wam1, self.c_pwd)
        self.assertFalse("Raymond E. Feist" in data_signed)
        result = self.c_cert.verify_smime(data_signed)
        self.assertTrue(result)
        data_signed = self.c_cert.sign_model(wam1, self.c_pwd, use_natural_keys=True)
        self.assertTrue("Raymond E. Feist" in data_signed)
        result = self.c_cert.verify_smime(data_signed)
        self.assertTrue(result)

    def testBasicModelSignature(self):
        """Try to sign a basic model and get a Signature
        """
        # Sign
        auth1 = Author(name="Raymond E. Feist", title="MR")
        auth1.save()
        self.c_cert.save()
        signed = self.c_cert.make_signature(auth1, self.c_pwd)
        self.assertTrue(isinstance(signed, Signature))
        content_type = ContentType.objects.get_for_model(auth1)
        self.assertEqual(signed.content_type, content_type)
        self.assertEqual(signed.object_id, 1)
        signed.save()

        # Verify
        signed = Signature.objects.get(pk=1)
        result = signed.check_pkcs7()
        self.assertTrue(result)
        result = signed.check()
        self.assertTrue(result)
        auth1.name = "JR"
        auth1.save()
        signed = Signature.objects.get(pk=1)
        result = signed.check()
        self.assertFalse(result)

    def testFileModelSignature(self):
        """Try to sign a basic model and get a Signature
        """
        # Sign
        filepath = os.path.join(settings.MEDIA_ROOT, 'afile.txt')
        book1 = Book(name="A book", afile=filepath)
        book1.save()
        book1 = Book.objects.get(pk=1)
        signed = self.c_cert.make_signature(book1, self.c_pwd)
        self.assertTrue(isinstance(signed, Signature))
        content_type = ContentType.objects.get_for_model(book1)
        self.assertEqual(signed.content_type, content_type)
        self.assertEqual(signed.object_id, 1)
        signed.save()

        # Verify
        signed = Signature.objects.get(pk=1)
        result = signed.check_pkcs7()
        self.assertTrue(result)
        result = signed.check()
        self.assertTrue(result)

        # other file with same content
        filepath = os.path.join(settings.MEDIA_ROOT, 'otherfile.txt')
        book1.afile = filepath
        book1.save()
        signed = Signature.objects.get(pk=1)
        result = signed.check_pkcs7()
        self.assertTrue(result)
        result = signed.check()
        self.assertTrue(result)

        # other file with wrong content
        afile = "wrongfile.txt"
        filepath = os.path.join(settings.MEDIA_ROOT, 'wrongfile.txt')
        book1.afile = filepath
        book1.save()
        signed = Signature.objects.get(pk=1)
        result = signed.check()
        self.assertFalse(result)

##################################
# Following tests are just for
# Code practice with openssl
# or M2Crypto
# They will be deleted
##################################

def getoutput(cmd, stdin=PIPE):
    cmd = shlex.split(cmd)
    a =  Popen(cmd, stdout=PIPE, stderr=PIPE, stdin=stdin).communicate()
    #print a[1]
    return a[0]


class RawTestCase(TestCase):
    """Tests with openssl lib

    Just for practice
    """
    def testCertificateGeneration(self):
        """Test SSL certificate generation
        """
        return
        # Pipe CA password
        CA_pwd = "toto"
        pCA_pwd = TemporaryFile()
        pCA_pwd.write(CA_pwd)
        pCA_pwd.seek(0)
        # Pipe User Password
        user_pwd = "tata"
        puser_pwd = TemporaryFile()
        puser_pwd.write(CA_pwd)
        puser_pwd.seek(0)

        # Generate CA's private key
        #
        # t = CA privkey
        t = NamedTemporaryFile()
        # We don't use tempfile for stdout because we want a variable
        # private_key for django_storage
        private_key = getoutput("openssl genrsa 4096")
        t.write(private_key)
        t.seek(0)

        # Generate Self signed RootCA
        #
        # TODO : generate and use temp openssl.cnf file
        #
        # Keep private key open while generating selfsigned certificate
        root_CA = getoutput("openssl req -new -x509 -batch -passout stdin -key %s" % t.name, stdin=pCA_pwd)

        # Generate user private key
        #
        # t2 = User private key
        t2 = NamedTemporaryFile()
        # We don't use tempfile for stdout because we want a variable
        # private_key for django_storage
        user_private_key = getoutput("openssl genrsa 4096")
        t2.write(private_key)
        t2.seek(0)

        # Generate Sign request
        #
        # TODO : generate and use temp openssl.cnf file
        #
        # t2 = User private key
        rqst = getoutput("openssl req -new -batch -passout stdin -key %s" % t2.name, stdin=puser_pwd)

        # Sign request
        #
        # TODO : 
        #   * generate and use temp openssl.cnf file
        #   * Verify informations
        #
        # Keep private key open while generating selfsigned certificate
        # t = CA privkey
        # t3 = request
        # t4 = root_CA
        # s = SerialFile
        t3 = NamedTemporaryFile()
        t3.write(rqst)
        t3.seek(0)
        t4 = NamedTemporaryFile()
        t4.write(root_CA)
        t4.seek(0)
        t.seek(0)
        pCA_pwd.seek(0)
        s = NamedTemporaryFile()
        s.write("02")
        s.seek(0)
        # subject = '/countryName=France/stateOrProvinceName=Ile-de-france/localityName=Paris/organizationName=Django-signature/organizationalUnitName=django-signature/commonName=iamauser'
        certificate = getoutput("openssl x509 -req -days 730 -passin stdin -in %s -CA %s -CAkey %s -CAserial %s" % (t3.name, t4.name, t.name, s.name), stdin=pCA_pwd)

        # Get public key from private key
        #
        # pcertificate = certificate
        # t2 = User private key
        t2.seek(0)
        pubkey = getoutput("openssl rsa -pubout -in %s" % t2.name)

        # Sign Something with certificate
        #
        # t2 = User private key
        pmy_text = NamedTemporaryFile()
        pmy_text.write(root_CA)
        pmy_text.seek(0)
        t2.seek(0)
        my_text = "Something really interesting"
        signature = getoutput("openssl dgst -sha1 -sign %s -hex -c" % (t2.name), stdin=pmy_text)

        # Verify text
        #
        # ppubkey = pubkey
        # XXX : FAIL !!!
        pmy_text.seek(0)
        psignature = NamedTemporaryFile()
        psignature.write(signature)
        psignature.seek(0)
        ppubkey = NamedTemporaryFile()
        ppubkey.write(pubkey)
        ppubkey.seek(0)
        verify = getoutput("openssl dgst -sha1 -verify %s -hex -signature %s" % (ppubkey.name, psignature.name), stdin=pmy_text)
        self.assertTrue("OK" in verify)
        # XXX : FAIL !!!

class M2TestCase(TestCase):
    """Tests with m2Crypto lib

    http://chandlerproject.org/bin/view/Projects/MeTooCrypto

    Theses tests are built with help of M2Crypto's testsuite
    """
    def quiet_callback(*args):
        return

    def testCertificateGeneration(self):
        """Test SSL certificate generation
        """
        CA_pwd = "toto"
        user_pwd = "tata"
        my_text = "Something really interesting"
        before = datetime(2000, 01, 01)
        after = datetime(2015, 01, 01)

        # Generate Self Signed CA #
        #
        # Generate CA Key
        ca_keys = RSA.gen_key(4096, 0x10001, callback=self.quiet_callback)
        ca_pkey = EVP.PKey(md='sha1')
        ca_pkey.assign_rsa(ca_keys)

        # Generate CA Request
        rqst = X509.Request()
        ca_name = rqst.get_subject()
        ca_name.C = "FR"
        ca_name.CN = "Certificate Auth"
        rqst.set_pubkey(ca_pkey)
        # Sign request
        rqst.sign(pkey=ca_pkey, md='sha1')
        #print rqst.as_text()

        # Make CA's self-signed certificate with CA request
        ca_cert = X509.X509()
        ca_cert.set_version(2)
        # Set certificate expiration
        asn1 = ASN1.ASN1_UTCTIME()
        asn1.set_datetime(before)
        ca_cert.set_not_before(asn1)
        asn1 = ASN1.ASN1_UTCTIME()
        asn1.set_datetime(after)
        ca_cert.set_not_after(asn1)
        # Use CA pubkey
        ca_cert.set_pubkey(ca_pkey)
        # Self signed : subject = issuer
        ca_cert.set_subject_name(ca_name)
        ca_cert.set_issuer_name(ca_name)
        # Add CA Constraint
        ext = X509.new_extension('basicConstraints', 'CA:TRUE')
        ca_cert.add_ext(ext)
        # Sign CA with CA's privkey
        ca_cert.sign(ca_pkey, md='sha1')
        #print "CA"
        #print ca_cert.as_text()
        self.assertTrue(ca_cert.check_ca())

        # Make client Certificate #
        #
        # Generate Client Key
        c_keys = RSA.gen_key(4096, 0x10001, callback=self.quiet_callback)
        c_pkey = EVP.PKey(md='sha1')
        c_pkey.assign_rsa(c_keys)

        # Generate client Request
        rqst = X509.Request()
        rqst_name = rqst.get_subject()
        rqst_name.C = "FR"
        rqst_name.CN = "Client to Auth"
        rqst_name.O = "My Big Company"
        rqst.set_pubkey(c_pkey)
        rqst.set_subject_name(rqst_name)
        # Sign request
        rqst.sign(pkey=c_pkey, md='sha1')
        #print "Client"
        #print rqst.as_text()

        # Make Client certificate with CA's key
        c_cert = X509.X509()
        c_cert.set_version(2)
        c_cert.set_serial_number(0)
        # Set certificate expiration
        asn1 = ASN1.ASN1_UTCTIME()
        asn1.set_datetime(before)
        c_cert.set_not_before(asn1)
        asn1 = ASN1.ASN1_UTCTIME()
        asn1.set_datetime(after)
        c_cert.set_not_after(asn1)
        # Get subject from request
        sub = rqst.get_subject()
        c_cert.set_subject(sub)
        # use client pubkey
        c_cert.set_pubkey(c_pkey)
        # issuer = CA
        ca_name = X509.X509_Name()
        ca_name.C = "FR"
        ca_name.CN = "Certificate Auth"
        c_cert.set_issuer(ca_name)
        # Sign Certificate with CA key
        c_cert.sign(ca_pkey, md='sha1')
        #print "Signed"
        #print c_cert.as_text()

        # Somes checks with openssl
        c_cert_temp = NamedTemporaryFile()
        ca_cert_temp = NamedTemporaryFile()
        c_cert.save_pem(c_cert_temp.name)
        ca_cert.save_pem(ca_cert_temp.name)
        # Verifiy CA with itself
        verify = getoutput("openssl verify -CAfile %s %s" % (ca_cert_temp.name, ca_cert_temp.name))
        #print verify
        self.assertTrue("OK" in verify)
        # Verifiy client cert with CA
        verify = getoutput("openssl verify -CAfile %s %s" % (ca_cert_temp.name, c_cert_temp.name))
        #print verify
        self.assertTrue("OK" in verify)
        #from M2Crypto.util import no_passphrase_callback
        #c_keys.save_key("/home/cyberj/tmp/ssl/c_pkey.pem", cipher=None, callback=no_passphrase_callback)
        #c_keys.save_key_bio(biog, cipher=None, callback=no_passphrase_callback)
        #ca_pkey.save_key("/home/cyberj/tmp/ssl/ca_pkey.pem", cipher=None, callback=no_passphrase_callback)

        # OK, certificate are generated and are OK
        #
        # We have :
        # ca_pkey = CA pub/Private Key
        # ca_cert = CA Certificate
        # c_pkey = Client pub/Private key
        # c_cert = Client Certificate signed by CA root
        #
        # It's time to sign/verify
        # reminder : http://sandbox.rulemaker.net/ngps/m2/howto.smime.html
        # reminder : http://svn.osafoundation.org/m2crypto/trunk/M2Crypto/SMIME.py

        from M2Crypto import BIO, SMIME
        text = "This is a data"
        # Set context
        s = SMIME.SMIME()
        s.pkey = c_pkey
        s.x509 = c_cert
        # Sign
        buf = BIO.MemoryBuffer(text)
        p7 = s.sign(buf, SMIME.PKCS7_DETACHED)
        # write content + signature
        out = BIO.MemoryBuffer()
        s.write(out, p7, BIO.MemoryBuffer(text))
        # get data signed
        data_signed = out.read()
        # data_signed is like : http://friendpaste.com/3fi7Ub8c2jYLxUR2f7wQ68
        # Save signed for tests
        #f = open("/home/cyberj/tmp/ssl/signed.mime", 'w')
        #f.write(data_signed)
        #f.close()

        # Check
        #print "Check"
        s = SMIME.SMIME()
        # Adds client crt
        sk = X509.X509_Stack()
        sk.push(c_cert)
        s.set_x509_stack(sk)
        # Adds CA crt
        st = X509.X509_Store()
        st.add_x509(ca_cert)
        s.set_x509_store(st)
        # Get data and p7 from data_signed
        bio_data_signed = BIO.MemoryBuffer(data_signed)
        p7, data = SMIME.smime_load_pkcs7_bio(bio_data_signed)
        #print data
        #print p7
        bio_data_wrong = BIO.MemoryBuffer("This is a duta")
        bio_data = BIO.MemoryBuffer("This is a data")
        verified = s.verify(p7, bio_data)
        #print verified
        self.assertRaises(SMIME.PKCS7_Error, s.verify, p7, bio_data_wrong)




#!/usr/bin/python

from os.path import abspath, dirname, join

CERT_ROOT = abspath(dirname(__file__))

CA_KEY_PATH=join(CERT_ROOT, 'rootca.key')
CA_KEY=open(CA_KEY_PATH, 'r').read()

C_KEY_PATH=join(CERT_ROOT, 'client.key')
C_KEY=open(C_KEY_PATH, 'r').read()

U_KEY_PATH=join(CERT_ROOT, 'user.key')
U_KEY=open(U_KEY_PATH, 'r').read()

C_PUB_KEY_PATH=join(CERT_ROOT, 'client.pub')
C_PUB_KEY=open(C_PUB_KEY_PATH, 'r').read()

CA_CERT_PATH=join(CERT_ROOT, 'rootca.crt')
CA_CERT=open(CA_CERT_PATH, 'r').read()

C_REQUEST_PATH=join(CERT_ROOT, 'client_req.pem')
C_REQUEST=open(C_REQUEST_PATH, 'r').read()

C_CERT_PATH=join(CERT_ROOT, 'client.crt')
C_CERT=open(C_CERT_PATH, 'r').read()

U_REQUEST_PATH=join(CERT_ROOT, 'user_req.pem')
U_REQUEST=open(U_REQUEST_PATH, 'r').read()

U_CERT_PATH=join(CERT_ROOT, 'user.crt')
U_CERT=open(U_CERT_PATH, 'r').read()

UTF8_CERT_PATH=join(CERT_ROOT, 'utf8.pem')
UTF8_CERT=open(UTF8_CERT_PATH, 'r').read()



import os
DEBUG = True
DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = '/tmp/extjs.db'
INSTALLED_APPS = (
    'django.contrib.auth', 
    'django.contrib.contenttypes', 
    'django.contrib.sessions', 
    'django.contrib.sites',
    'signature',
    'test_project.apps.testapp',
)
TEMPLATE_DIRS = (
    os.path.join(os.path.dirname(__file__), 'templates'),
)

SITE_ID = 1
ROOT_URLCONF = 'test_project.urls'

PROJECT_PATH = os.path.abspath(os.path.split(__file__)[0])
MEDIA_ROOT = os.path.join(PROJECT_PATH, 'media')



from django.db import models
from django.utils.translation import ugettext_lazy as _

class Author(models.Model):
    TITLE_CHOICES = (
            ('MR', _('Mr.')),
            ('MRS', _('Mrs.')),
            ('MS', _('Ms.')),
    )
    name = models.CharField(_('Name'), max_length=100, default="Platon", unique=True)
    title = models.CharField(max_length=3, choices=TITLE_CHOICES)
    birth_date = models.DateField(blank=True, null=True)

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)


class Whatamess(models.Model):
    TITLE_CHOICES = (
            (1, _('Mr.')),
            (2, _('Mrs.')),
            (3, _('Ms.')),
    )
    name = models.CharField(max_length=100)
    number = models.IntegerField()
    slug = models.SlugField()
    text = models.TextField()
    author = models.ForeignKey(Author, null=True)
    title = models.PositiveSmallIntegerField(max_length=3, choices=TITLE_CHOICES)
    birth_date = models.DateTimeField(blank=True, null=True)
    yesno = models.BooleanField()

    def __unicode__(self):
        return self.name

    def yes(self):
        return True

class AuthorProxy(Author):
        class Meta:
            proxy = True

        @property
        def aprint(self):
            return "I'm a proxy : %s" % self.name

class Book(models.Model):
    name = models.CharField(_('Name'), max_length=100, default="The Hitchhiker's Guide to the Galaxy", unique=True)
    afile = models.FileField(upload_to=".")
    #apicture = models.ImageField(upload_to="/", blank=True, null=True)

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)



from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_str, smart_unicode
from M2Crypto import BIO, m2, ASN1, RSA, EVP, X509, SMIME
from M2Crypto.util import no_passphrase_callback
from signature import utils
from tempfile import NamedTemporaryFile, TemporaryFile
from signature.openssl import Openssl

from datetime import datetime

COUNTRY = (
    ('AD', 'AD'),('AE', 'AE'),('AF', 'AF'),('AG', 'AG'),('AI', 'AI'),('AL', 'AL'),('AM', 'AM'),
    ('AN', 'AN'),('AO', 'AO'),('AQ', 'AQ'),('AR', 'AR'),('AS', 'AS'),('AT', 'AT'),('AU', 'AU'),
    ('AW', 'AW'),('AZ', 'AZ'),('BA', 'BA'),('BB', 'BB'),('BD', 'BD'),('BE', 'BE'),('BF', 'BF'),
    ('BG', 'BG'),('BH', 'BH'),('BI', 'BI'),('BJ', 'BJ'),('BM', 'BM'),('BN', 'BN'),('BO', 'BO'),
    ('BR', 'BR'),('BS', 'BS'),('BT', 'BT'),('BU', 'BU'),('BV', 'BV'),('BW', 'BW'),('BY', 'BY'),
    ('BZ', 'BZ'),('CA', 'CA'),('CC', 'CC'),('CF', 'CF'),('CG', 'CG'),('CH', 'CH'),('CI', 'CI'),
    ('CK', 'CK'),('CL', 'CL'),('CM', 'CM'),('CN', 'CN'),('CO', 'CO'),('CR', 'CR'),('CS', 'CS'),
    ('CU', 'CU'),('CV', 'CV'),('CX', 'CX'),('CY', 'CY'),('CZ', 'CZ'),('DD', 'DD'),('DE', 'DE'),
    ('DJ', 'DJ'),('DK', 'DK'),('DM', 'DM'),('DO', 'DO'),('DZ', 'DZ'),('EC', 'EC'),('EE', 'EE'),
    ('EG', 'EG'),('EH', 'EH'),('ER', 'ER'),('ES', 'ES'),('ET', 'ET'),('FI', 'FI'),('FJ', 'FJ'),
    ('FK', 'FK'),('FM', 'FM'),('FO', 'FO'),('FR', 'FR'),('FX', 'FX'),('GA', 'GA'),('GB', 'GB'),
    ('GD', 'GD'),('GE', 'GE'),('GF', 'GF'),('GH', 'GH'),('GI', 'GI'),('GL', 'GL'),('GM', 'GM'),
    ('GN', 'GN'),('GP', 'GP'),('GQ', 'GQ'),('GR', 'GR'),('GS', 'GS'),('GT', 'GT'),('GU', 'GU'),
    ('GW', 'GW'),('GY', 'GY'),('HK', 'HK'),('HM', 'HM'),('HN', 'HN'),('HR', 'HR'),('HT', 'HT'),
    ('HU', 'HU'),('ID', 'ID'),('IE', 'IE'),('IL', 'IL'),('IN', 'IN'),('IO', 'IO'),('IQ', 'IQ'),
    ('IR', 'IR'),('IS', 'IS'),('IT', 'IT'),('JM', 'JM'),('JO', 'JO'),('JP', 'JP'),('KE', 'KE'),
    ('KG', 'KG'),('KH', 'KH'),('KI', 'KI'),('KM', 'KM'),('KN', 'KN'),('KP', 'KP'),('KR', 'KR'),
    ('KW', 'KW'),('KY', 'KY'),('KZ', 'KZ'),('LA', 'LA'),('LB', 'LB'),('LC', 'LC'),('LI', 'LI'),
    ('LK', 'LK'),('LR', 'LR'),('LS', 'LS'),('LT', 'LT'),('LU', 'LU'),('LV', 'LV'),('LY', 'LY'),
    ('MA', 'MA'),('MC', 'MC'),('MD', 'MD'),('MG', 'MG'),('MH', 'MH'),('ML', 'ML'),('MM', 'MM'),
    ('MN', 'MN'),('MO', 'MO'),('MP', 'MP'),('MQ', 'MQ'),('MR', 'MR'),('MS', 'MS'),('MT', 'MT'),
    ('MU', 'MU'),('MV', 'MV'),('MW', 'MW'),('MX', 'MX'),('MY', 'MY'),('MZ', 'MZ'),('NA', 'NA'),
    ('NC', 'NC'),('NE', 'NE'),('NF', 'NF'),('NG', 'NG'),('NI', 'NI'),('NL', 'NL'),('NO', 'NO'),
    ('NP', 'NP'),('NR', 'NR'),('NT', 'NT'),('NU', 'NU'),('NZ', 'NZ'),('OM', 'OM'),('PA', 'PA'),
    ('PE', 'PE'),('PF', 'PF'),('PG', 'PG'),('PH', 'PH'),('PK', 'PK'),('PL', 'PL'),('PM', 'PM'),
    ('PN', 'PN'),('PR', 'PR'),('PT', 'PT'),('PW', 'PW'),('PY', 'PY'),('QA', 'QA'),('RE', 'RE'),
    ('RO', 'RO'),('RU', 'RU'),('RW', 'RW'),('SA', 'SA'),('SB', 'SB'),('SC', 'SC'),('SD', 'SD'),
    ('SE', 'SE'),('SG', 'SG'),('SH', 'SH'),('SI', 'SI'),('SJ', 'SJ'),('SK', 'SK'),('SL', 'SL'),
    ('SM', 'SM'),('SN', 'SN'),('SO', 'SO'),('SR', 'SR'),('ST', 'ST'),('SU', 'SU'),('SV', 'SV'),
    ('SY', 'SY'),('SZ', 'SZ'),('TC', 'TC'),('TD', 'TD'),('TF', 'TF'),('TG', 'TG'),('TH', 'TH'),
    ('TJ', 'TJ'),('TK', 'TK'),('TM', 'TM'),('TN', 'TN'),('TO', 'TO'),('TP', 'TP'),('TR', 'TR'),
    ('TT', 'TT'),('TV', 'TV'),('TW', 'TW'),('TZ', 'TZ'),('UA', 'UA'),('UG', 'UG'),('UM', 'UM'),
    ('US', 'US'),('UY', 'UY'),('UZ', 'UZ'),('VA', 'VA'),('VC', 'VC'),('VE', 'VE'),('VG', 'VG'),
    ('VI', 'VI'),('VN', 'VN'),('VU', 'VU'),('WF', 'WF'),('WS', 'WS'),('YD', 'YD'),('YE', 'YE'),
    ('YT', 'YT'),('YU', 'YU'),('ZA', 'ZA'),('ZM', 'ZM'),('ZR', 'ZR'),('ZW', 'ZW'),('ZZ', 'ZZ'),
    ('ZZ', 'ZZ'),
    )

class BaseCert(models.Model):
    """Base Certificate for Models
    """
    key = models.ForeignKey('Key', null=True)
    pem = models.TextField(editable=False)
    country = models.CharField(max_length=2, choices=COUNTRY)
    state  = models.CharField(max_length=32, null=True)
    locality = models.CharField(max_length=32, null=True)
    organization = models.CharField(max_length=64, null=True)
    created = models.DateTimeField()
    CN = models.CharField(max_length=50)
    OU = models.CharField(max_length=50, null=True)
    email = models.EmailField(blank=True, null=True)
    user = models.ForeignKey(User, null=True)
    class Meta:
        abstract = True

    def get_subject(self):
        """Return subject string for CSR and self-signed certs
        """

        subj = '/CN=%s' % self.CN

        if self.country:
            subj += '/C=%s' % self.country
        if self.state:
            subj += '/ST=%s' % self.state
        if self.locality:
            subj += '/localityName=%s' % self.locality
        if self.organization:
            subj += '/O=%s' % self.organization
        if self.OU:
            subj += '/organizationalUnitName=%s' % self.OU
        if self.email:
            subj += '/emailAddress=%s' % self.email
        return subj

    def get_pubkey(self):
        """Retrieve pubkey of certificate
        """
        bio = BIO.MemoryBuffer()
        self.m2_x509().get_pubkey().get_rsa().save_pub_key_bio(bio)
        return bio.read()

def quiet_callback(*args):
        return

def quiet_passphrase(passphrase=None):
    if passphrase == None:
        ciph = None
        cb = no_passphrase_callback
    else:
        ciph = 'aes_128_cbc'
        cb = lambda a: passphrase
    return ciph, cb

class Key(models.Model):
    """An Private/public key pair

    TODO : RSA/EVP ??
    """
    user = models.ForeignKey(User, null=True)
    length = models.PositiveIntegerField(editable=False)
    private = models.TextField(editable=False)
    public = models.TextField(editable=False)

    @classmethod
    def generate(cls, passphrase, length=4096, user=None):
        """Return an Key instance with RSA key generated
        """
        ciph, cb = quiet_passphrase(passphrase)
        keys = RSA.gen_key(length, 0x10001, callback=quiet_callback)
        key = cls()
        key.length = length
        bio = BIO.MemoryBuffer()
        keys.save_pub_key_bio(bio)
        key.public = bio.read()
        keys.save_key_bio(bio, cipher=ciph, callback=cb)
        key.private = bio.read()
        return key

    def m2_rsa(self, passphrase=None):
        """Return M2Crypto RSA's instance of key
        """
        ciph, cb = quiet_passphrase(passphrase)
        rsakeyp = RSA.load_key_string(smart_str(self.private), cb)
        return rsakeyp

    def m2_pkey(self, passphrase=None):
        """Return M2Crypto EVP's instance of key
        """
        rsakeyp = self.m2_rsa(passphrase)
        ciph, cb = quiet_passphrase(passphrase)
        evp_pkey = EVP.PKey(md='sha1')
        evp_pkey.assign_rsa(rsakeyp)
        return evp_pkey

    @classmethod
    def new_from_pem(cls, pem, passphrase=None, user=None):
        """Create a Key Instance with an existing PEM
        """
        ciph, cb = quiet_passphrase(passphrase)
        key = cls(user=user)
        m2key = RSA.load_key_string(pem, cb)
        key.private = m2key.as_pem(ciph, cb)
        bio = BIO.MemoryBuffer()
        m2key.save_pub_key_bio(bio)
        key.public = bio.read()
        key.length = len(m2key)

        key.save()
        # Find Relations
        if user:
            for cert in Certificate.objects.filter(user=user, key__isnull=True):
                if cert.get_pubkey() == key.public:
                    cert.key = key
                    cert.save()
            for rqst in CertificateRequest.objects.filter(user=user, key__isnull=True):
                if rqst.get_pubkey() == key.public:
                    rqst.key = key
                    rqst.save()
        else:
            for cert in Certificate.objects.filter(user__isnull=True, key__isnull=True):
                if cert.get_pubkey() == key.public:
                    cert.key = key
                    cert.save()
            for rqst in CertificateRequest.objects.filter(user__isnull=True, key__isnull=True):
                if rqst.get_pubkey() == key.public:
                    rqst.key = key
                    rqst.save()
        return key

class CertificateRequest(BaseCert):
    """A CSR
    """

    def m2_request(self):
        """Return M2Crypto's Request instance
        """
        rqst = X509.load_request_string(self.pem, X509.FORMAT_PEM)
        return rqst

    def sign_request(self, passphrase=None):
        """Generate request with instance informations
        """
        # TODO : class for C / CN and all attributes
        # Generate CA Request
        rqst = X509.Request()
        issuer_name = rqst.get_subject()
        issuer_name.CN = self.CN
        if self.country:
            issuer_name.C = self.country
        if self.locality:
            issuer_name.L = self.locality
        if self.organization:
            issuer_name.O = self.organization
        if self.OU:
            issuer_name.OU = self.OU
        if self.state:
            issuer_name.SP = self.state
        if self.email:
            issuer_name.Email = self.email
        issuer_pkey = self.key.m2_pkey(passphrase)
        rqst.set_pubkey(issuer_pkey)
        rqst.sign(pkey=issuer_pkey, md='sha1')
        # Add date
        self.created = datetime.now()
        self.pem = rqst.as_pem()

    @classmethod
    def new_from_pem(cls, pem, user=None, key=None):
        """Create a Request Instance with an existing PEM
        """
        rqst = cls(user=user, key=key)
        m2rqst = X509.load_request_string(pem, X509.FORMAT_PEM)
        rqst.pem = m2rqst.as_pem()
        subject = m2rqst.get_subject()
        rqst.country = subject.C
        rqst.CN = subject.CN
        # Add date
        rqst.created = datetime.now()
        return rqst

    def __unicode__(self):
        return "CSR %s" % self.get_subject()

class Signature(models.Model):
    """A PKCS#7 signature for a model
    """
    certificate = models.ForeignKey('Certificate')
    object_id = models.PositiveIntegerField(null=True)
    content_type = models.ForeignKey(ContentType, null=True)
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    pkcs7 = models.TextField()

    def check_pkcs7(self):
        """Check PKCS7 signature
        (don't compare with original model)
        """
        pkcs7 = smart_str(self.pkcs7)
        return self.certificate.verify_smime(pkcs7)

    def check(self):
        """Check Signature
        """
        pkcs7_data = self.check_pkcs7()
        if not pkcs7_data:
            return False
        # TODO : That is ugly !
        pkcs7_data = pkcs7_data.replace("\r\n", "\n")
        serialized = utils.serialize(self.content_object)
        return serialized == pkcs7_data

class Certificate(BaseCert):
    """An x509 certificate
    """
    begin = models.DateTimeField(editable=False)
    end = models.DateTimeField(editable=False)
    days = models.IntegerField(null=True)
    serial = models.CharField(max_length=21, editable=False)
    issuer = models.ForeignKey('self', related_name='issuer_set', null=True)
    is_ca = models.BooleanField(default=False)
    ca_serial = models.PositiveIntegerField(null=True, editable=False)
    subject_kid = models.CharField(max_length=60, editable=False)
    auth_kid = models.CharField(max_length=60, editable=False)
    crl = models.TextField(editable=False, null=True, blank=True)
    crlnumber = models.PositiveIntegerField(editable=False, null=True, blank=True)
    revoked = models.BooleanField(editable=False, default=False)
    trust = models.NullBooleanField(editable=False, null=True)
    certhash = models.CharField(editable=False, max_length=9)
    index = models.TextField(editable=False, default="") # temporary

    def __unicode__(self):
        if self.is_ca:
            return "x509 CA %s" % self.get_subject()
        else:
            return "x509 %s" % self.get_subject()

    class Meta:
        unique_together = (("subject_kid", "serial"))

    def m2_x509(self):
        """Return M2Crypto's x509 instance of certificate
        """
        pem = smart_str(self.pem)
        cert = X509.load_cert_string(pem, X509.FORMAT_PEM)
        return cert

    def generate_x509_root(self, passphrase=None):
        """Generate x509 certificate with instance informations
        """
        # Generate CA Request
        ca_pkey = self.key.private

        subject = self.get_subject()

        ossl = Openssl()
        pem = ossl.generate_self_signed_cert(self.days, subject, ca_pkey, passphrase)
        self.pem = pem
        self.certhash = ossl.get_hash_from_cert(pem)
        x509 = X509.load_cert_string(pem, X509.FORMAT_PEM)
        self.serial = str(x509.get_serial_number())
        self.begin = x509.get_not_before().get_datetime()
        self.end = x509.get_not_after().get_datetime()
        # v3 extensions
        self.subject_kid = x509.get_ext("subjectKeyIdentifier").get_value().strip()
        auth_kid = x509.get_ext("authorityKeyIdentifier").get_value().split("\n")
        self.auth_kid = [keyid.lstrip('keyid:') for keyid in auth_kid if keyid.startswith("keyid:")][0].strip()
        self.ca_serial = 1
        self.is_ca = True
        self.trust = True
        # Add date
        self.created = datetime.now()

    def sign_request(self, rqst, days, passphrase=None, ca=False):
        """Sign a Request and return a Certificate instance
        """
        ossl = Openssl()

        pem = ossl.sign_csr(rqst.pem, self.key.private, self.pem, self.ca_serial, days, passphrase, ca)
        self.ca_serial += 1
        self.save()

        c_cert = Certificate()
        c_cert.pem = pem
        c_cert.certhash = ossl.get_hash_from_cert(pem)
        c_cert.user = rqst.user
        c_cert.issuer = self
        c_cert.key = rqst.key
        c_cert.country = rqst.country
        c_cert.CN = rqst.CN
        c_cert.locality = rqst.locality
        c_cert.email = rqst.email
        c_cert.organization = rqst.organization
        c_cert.OU = rqst.OU
        c_cert.state = rqst.state

        x509 = X509.load_cert_string(pem, X509.FORMAT_PEM)
        c_cert.serial = str(x509.get_serial_number())
        c_cert.begin = x509.get_not_before().get_datetime()
        c_cert.end = x509.get_not_after().get_datetime()
        # v3 extensions
        c_cert.subject_kid = x509.get_ext("subjectKeyIdentifier").get_value().strip()
        auth_kid = x509.get_ext("authorityKeyIdentifier").get_value().split("\n")
        c_cert.auth_kid = [keyid.lstrip('keyid:') for keyid in auth_kid if keyid.startswith("keyid:")][0].strip()
        if ca:
            c_cert.ca_serial = 1
            c_cert.is_ca = True
        # Add date
        c_cert.created = datetime.now()

        # And return new instance
        return c_cert

    @classmethod
    def new_from_pem(cls, pem, user=None, key=None):
        """Create a Certificate Instance with an existing PEM
        """
        ossl = Openssl()
        cert = cls(user=user, key=key)
        x509 = X509.load_cert_string(pem, X509.FORMAT_PEM)
        cert.pem = x509.as_pem()
        cert.certhash = ossl.get_hash_from_cert(pem)
        issuer = x509.get_issuer()
        if issuer.C:
            cert.country = smart_unicode(issuer.C)
        cert.CN = smart_unicode(issuer.CN)
        if issuer.L:
            cert.locality = smart_unicode(issuer.L)
        if issuer.Email:
            cert.email = smart_unicode(issuer.Email)
        if issuer.O:
            cert.organization = smart_unicode(issuer.O)
        if issuer.OU:
            cert.OU = smart_unicode(issuer.OU)
        if issuer.SP:
            cert.state = smart_unicode(issuer.SP)
        cert.serial = str(x509.get_serial_number())
        cert.begin = x509.get_not_before().get_datetime()
        cert.end = x509.get_not_after().get_datetime()
        # v3 extensions
        cert.subject_kid = x509.get_ext("subjectKeyIdentifier").get_value().strip()
        auth_kid = x509.get_ext("authorityKeyIdentifier").get_value().split("\n")
        cert.auth_kid = [keyid.lstrip('keyid:') for keyid in auth_kid if keyid.startswith("keyid:")][0].strip()

        # Search issuer
        try:
            ca_cert = Certificate.objects.get(subject_kid=cert.auth_kid)
        except Certificate.DoesNotExist:
            pass
        else:
            cert.issuer = ca_cert

        # Find Relations
        cert_pubkey = cert.get_pubkey()
        if key:
            cert.key = key
        else:
            if user:
                try:
                    key = Key.objects.get(user=user, public=cert_pubkey)
                except Key.DoesNotExist:
                    pass
                else:
                    cert.key = key
            else:
                try:
                    key = Key.objects.get(public=cert_pubkey)
                except Key.DoesNotExist:
                    pass
                else:
                    cert.key = key
                    cert.user = key.user
        # Add date
        cert.created = datetime.now()
        if x509.check_ca():
            cert.is_ca = True
        cert.save()

        # Search issued # XXX
        for c_cert in Certificate.objects.filter(auth_kid=cert.subject_kid, issuer__isnull=True):
            c_cert.issuer = cert
            c_cert.save()

        return cert

    def sign_text(self, text, passphrase):
        """Sign a text with cert's key and passphrase
        """
        if not self.key:
            raise Exception("No key for this certificate")

        ossl = Openssl()
        data_signed = ossl.sign_pkcs7(self.pem, text, self.key.private, passphrase)
        return data_signed

    def get_issued(self):
        """Retrieve all certificates issued by the certificate
        """
        chain = Certificate.objects.filter(issuer=self)
        return chain

    def get_cert_chain(self):
        """Retrieve all certificates of the certificate chain
        """
        chain = []
        current_cert = self
        while current_cert:
            chain.append(current_cert)
            current_cert = current_cert.issuer
        chain.reverse()
        return chain

    def get_cert_chain_aspem(self):
        """Retrieve all certificates of the certificate chain
        as pem
        """
        pemchain = []
        chain = self.get_cert_chain()
        for cert in chain:
            crlchain.append(cert.pem)
        return crlchain

    def get_crl_chain_aspem(self):
        """Retrieve all crls of the certificate chain
        """
        crlchain = []
        chain = self.get_cert_chain()
        for cert in chain:
            if cert.crl:
                crlchain.append(cert.crl)
        return crlchain

    def check_chain(self, chain=None, silent=False):
        """Check certificate chain
        """
        if not chain:
            chain = self.get_cert_chain()
        ossl = Openssl()
        result = False
        if silent:
            try:
                result = ossl.verify_ca_chain(chain)
            except ossl.VerifyError:
                return False
        else:
            result = ossl.verify_ca_chain(chain)
        return result

    def check_crl(self, silent=False, quick=False):
        """Check CRL for this certificate

        Quick only use database cache
        """
        if self.revoked:
            return False
        elif quick:
            return True
        if self.issuer:
            if self.issuer.crl:
                ossl = Openssl()
                return not ossl.get_revoke_status_from_cert(self, self.issuer.crl)
        elif self.subject_kid != self.auth_kid:
            return False
        return True

    def check_crl_chain(self, chain=None, silent=False, quick=False):
        """Check CRL chain

        Quick only use database cache
        """
        for cert in chain:
            if not cert.check_crl(silent, quick):
                return False
        return True

    def check(self, silent=False, quick=False, crlcheck=True):
        """Check certificate
        """
        chain = self.get_cert_chain()
        chain_valid = self.check_chain(chain, silent)
        crl_valid = False
        if crlcheck:
            crl_valid = self.check_crl_chain(chain, silent, quick)
        return chain_valid and (crl_valid or not crlcheck)

    def gen_crl(self, passphrase=""):
        """Generate CRL for this certificate
        """
        cakey = self.key.private
        issued = self.get_issued()
        crlnumber = self.crlnumber or 1
        ossl = Openssl()
        self.crl = ossl.generate_crl(self, cakey, crlnumber, self.crl, issued, passphrase=passphrase)
        self.crlnumber = crlnumber + 1
        self.save()
        return self.crl

    def revoke(self, cert, passphrase=""):
        """Generate CRL for this certificate
        """
        if cert.issuer != self:
            raise Exception("I'm not the issuer.")

        cakey = self.key.private
        issued = self.get_issued()
        crlnumber = self.crlnumber or 1
        ossl = Openssl()
        self.crl, self.index = ossl.revoke_cert(self, cakey, crlnumber, self.crl, cert, issued, passphrase=passphrase)
        self.crlnumber = crlnumber + 1
        self.save()
        cert.revoked = True
        cert.save()
        return self.crl

    def sign_model(self, obj, passphrase, use_natural_keys=False, *args, **kwargs):
        """Sign a model instance or a queryset
        """
        serialized = utils.serialize(obj, use_natural_keys=use_natural_keys, *args, **kwargs)
        signed = self.sign_text(serialized, passphrase)
        return signed

    def verify_smime(self, smime, silent=False):
        """Verify an smime signed message
        """
        if not self.key:
            raise Exception("No key for this certificate")

        ossl = Openssl()
        if silent:
            try:
                data_signed = ossl.verify_pkcs7(self.pem, smime)
            except ossl.VerifyError:
                return False
        else:
            data_signed = ossl.verify_pkcs7(self.pem, smime)
        return data_signed

    def make_signature(self, instance, passphrase, fields=[], exclude=[]):
        """Sign a Model instance with passphrase
        """
        if not self.key:
            raise Exception("No key for this certificate")

        signature = Signature()

        signature.pkcs7 = self.sign_model(instance, passphrase)
        signature.certificate = self
        signature.content_type = ContentType.objects.get_for_model(instance)
        signature.object_id = instance.id
        return signature



from django.contrib import admin
from models import Key, CertificateRequest, Signature, Certificate

class KeyAdmin(admin.ModelAdmin):
    pass
admin.site.register(Key, KeyAdmin)

class CertificateRequestAdmin(admin.ModelAdmin):
    pass
admin.site.register(CertificateRequest, CertificateRequestAdmin)

class SignatureAdmin(admin.ModelAdmin):
    pass
admin.site.register(Signature, SignatureAdmin)

class CertificateAdmin(admin.ModelAdmin):
    pass
admin.site.register(Certificate, CertificateAdmin)



"""
From django-pki - Copyright (C) 2010 Daniel Kerwin <django-pki@linuxaddicted.de>
    http://github.com/dkerwin/django-pki

                - Copyright (C) 2010 Johan Charpentier <jcharpentier@bearstech.com>


This program and entire repository is free software; you can
redistribute it and/or modify it under the terms of the GNU
General Public License as published by the Free Software
Foundation; either version 2 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; If not, see <http://www.gnu.org/licenses/>.
"""
import os, re, sys
import datetime
import string, random

from signature.settings import PKI_OPENSSL_BIN, PKI_OPENSSL_CONF, PKI_DIR, PKI_OPENSSL_TEMPLATE, PKI_SELF_SIGNED_SERIAL

from subprocess import Popen, PIPE, STDOUT
import shutil
from logging import getLogger
from tempfile import NamedTemporaryFile, TemporaryFile, mkdtemp

try:
    # available in python-2.5 and greater
    from hashlib import md5 as md5_constructor
except ImportError:
    # compatibility fallback
    from md5 import new as md5_constructor

from django.template.loader import render_to_string
from django.utils.encoding import smart_str, smart_unicode

logger = getLogger("pki")

##------------------------------------------------------------------##
## OpenSSLConfig: Config related stuff
##------------------------------------------------------------------##

def refresh_pki_metadata(ca_list):
    """Refresh pki metadata (PKI storage directories and openssl configuration files)

    Each ca_list element is a dictionary:
    'name': CA name
    'subcas_allowed': sub CAs allowed (boolean)
    """

    status = True

    # refresh directory structure
    dirs = { 'certs'  : 0755,
             'private': 0700,
             'crl'    : 0755,
           }

    try:
        # create base PKI directory if necessary
        if not os.path.exists(PKI_DIR):
            logger.info('Creating base PKI directory')
            os.mkdir(PKI_DIR, 0700)

        # list of old CA directories for possible purging
        purge_dirs = set([os.path.join(PKI_DIR, d) for d in os.listdir(PKI_DIR)
                          if os.path.isdir(os.path.join(PKI_DIR, d))])

        # loop over CAs and create necessary filesystem objects
        for ca in ca_list:
            ca_dir = os.path.join(PKI_DIR, ca.name)

            # create CA directory if necessary
            if not ca_dir in purge_dirs:
                logger.info('Creating base directory for CA %s' % ca.name)
                os.mkdir(ca_dir)

                # create nested directories for key storage with proper permissions
                for d, m in dirs.items():
                    os.mkdir(os.path.join(ca_dir, d), m)

                initial_serial = 0x01

                try:
                    if not ca.parent and int(PKI_SELF_SIGNED_SERIAL) > 0:
                        initial_serial = PKI_SELF_SIGNED_SERIAL+1
                except ValueError:
                    logger.error( "PKI_SELF_SIGNED_SERIAL failed conversion to int!" )

                h2s = '%X' % initial_serial

                if len(h2s) % 2 == 1:
                    h2s = '0' + h2s

                # initialize certificate serial number
                s = open(os.path.join(ca_dir, 'serial'), 'wb')
                s.write(h2s)
                s.close()

                # initialize CRL serial number
                s = open(os.path.join(ca_dir, 'crlnumber'), 'wb')
                s.write('01')
                s.close()

                # touch certificate index file
                open(os.path.join(ca_dir, 'index.txt'), 'wb').close()

            # do not delete existing CA dir
            purge_dirs.discard(ca_dir)

        # purge unused CA directories
        for d in purge_dirs:
            if os.path.isdir(d):
                # extra check in order to keep unrelated directory from recursive removal...
                # (in case if something wrong with paths)
                # probably can be removed when debugging will be finished
                if os.path.isfile(os.path.join(d, 'crlnumber')):
                    logger.debug("Purging CA directory tree %s" % d)
                    shutil.rmtree(d) # FIXME: commented for debugging purposes
                else:
                    logger.warning('Directory %s does not contain any metadata, preserving it' % d)

    except OSError, e: # FIXME: probably catch any exception here, not just OS
        status = False
        logger.error("Refreshing directory structure failed: %s" % e)

    # prepare context for template rendering
    ctx = {'ca_list': ca_list}

    # render template and save result to openssl.conf
    try:
        conf = render_to_string(PKI_OPENSSL_TEMPLATE, ctx)

        f = open(PKI_OPENSSL_CONF, 'wb')
        f.write(conf)
        f.close()

    except:
        raise Exception( 'Failed to render OpenSSL template' )
        status = False

    return status # is it used somewhere?

##------------------------------------------------------------------##
## OpenSSLActions: All non config related actions
##------------------------------------------------------------------##

def in_temp_dir(func):
    """Create a temp dir and clean them on end of function
    """
    def new_func(*args, **kwargs):
        if args[0].tmpdir:
            return func(*args, **kwargs)
        tmpdir = mkdtemp()
        oldvalue = args[0].tmpdir
        args[0].tmpdir = tmpdir
        try:
            result = func(*args, **kwargs)
        finally:
            shutil.rmtree(tmpdir)
            args[0].tmpdir = oldvalue
        return result

    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    new_func.__dict__.update(func.__dict__)
    return new_func

class Openssl():
    '''Do the real openssl work - Generate keys, csr, sign'''

    def __init__(self, tmpdir=""):
        '''Class constructor'''

        ## Generate a random string as ENV variable name
        self.env_pw = "".join(random.sample(string.letters+string.digits, 10))

        #self.confname = PKI_OPENSSL_CONF
        self.tmpdir = tmpdir

    class VerifyError(Exception):
        """Openssl verify fails
        """
        pass

    def exec_rehash(directory):
        """call c_rehash on directory
        """
        proc = Popen("/usr/bin/c_rehash %s" % directory, shell=False, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
        stdout_value, stderr_value = proc.communicate()

        if proc.returncode != 0:
            #logger.error('openssl command "%s" failed with returncode %d' % (c[1], proc.returncode))
            #logger.error(stdout_value)

            raise self.VerifyError(stdout_value)
        else:
            return stdout_value

    def exec_openssl(self, command, stdin=None, env_vars=None, cwd=None):
        '''Run openssl command. PKI_OPENSSL_BIN doesn't need to be specified'''

        command = [smart_str(x) for x in command]
        stdin = smart_str(stdin)

        c = [PKI_OPENSSL_BIN]
        c.extend(command)

        # add PKI_DIR environment variable if caller did not set it
        if env_vars:
            env_vars.setdefault('PKI_DIR', PKI_DIR)
        else:
            env_vars = {'PKI_DIR': PKI_DIR}

        proc = Popen(c, shell=False, env=env_vars, stdin=PIPE, stdout=PIPE, stderr=STDOUT, cwd=cwd)
        stdout_value, stderr_value = proc.communicate(stdin)

        if proc.returncode != 0:
            #logger.error('openssl command "%s" failed with returncode %d' % (c[1], proc.returncode))
            #logger.error(stdout_value)

            raise self.VerifyError(stdout_value)
        else:
            return stdout_value

    def generate_self_signed_cert(self, days, subj, key, passphrase=None):
        """Generate a self signed root certificate
        """
        key_f = NamedTemporaryFile()
        key_f.write(key)
        key_f.seek(0)

        logger.info( 'Generating self-signed root certificate' )

        #command = ['req', '-config', self.confname, '-batch', '-sha1', '-new', '-x509', '-subj', subj, '-days', str(days), \
        command = ['req', '-batch', '-sha1', '-new', '-x509', '-subj', subj, '-days', str(days), \
                   '-extensions', 'v3_ca', '-key', key_f.name, '-passin', 'stdin']

        pem = self.exec_openssl(command, stdin=passphrase)
        return pem

    def generate_der_encoded(self):
        '''Generate a DER encoded version of a given certificate'''

        logger.info( 'Generating DER encoded certificate for %s' % self.i.name )

        command = 'x509 -in %s -out %s -outform DER' % (self.crt, self.der)
        self.exec_openssl(command.split())

        return True

    @in_temp_dir
    def sign_csr(self, csr, cakey, cacrt, serial, days, passphrase=None, ca_capable=False):
        """Sign the CSR with given CA
        """
        shutil.copy(PKI_OPENSSL_CONF, self.tmpdir)
        confpath = os.path.join(self.tmpdir, os.path.split(PKI_OPENSSL_CONF)[-1])
        logger.info( 'Signing CSR' )

        if ca_capable:
            extension = "v3_ca"
        else:
            extension = "usr_cert"
        csrfile = NamedTemporaryFile()
        csrfile.write(csr)
        csrfile.seek(0)
        cafile = NamedTemporaryFile()
        cafile.write(cacrt)
        cafile.seek(0)
        cakeyfile = NamedTemporaryFile()
        cakeyfile.write(cakey)
        cakeyfile.seek(0)
        serialfile = NamedTemporaryFile()
        serial = "%X" % serial
        serial = serial.rjust(2,"0")
        serialfile.write(serial)
        serialfile.seek(0)
        certfile = NamedTemporaryFile()

        command = ['x509', '-req', '-CAserial', serialfile.name,'-extfile', confpath , '-sha1', '-days', str(days), '-in', csrfile.name, '-CA', cafile.name, '-CAkey', cakeyfile.name, '-passin', 'stdin', '-extensions', extension, '-out', certfile.name]
        self.exec_openssl(command, stdin=passphrase)
        pem = certfile.read()
        return pem

    def _revoke_certificate(self, ppf):
        '''Revoke a given certificate'''

        ## Check if certificate is already revoked. May have happened during a incomplete transaction
        if self.get_revoke_status_from_cert():
            logger.info( "Skipping revoke as it already happened" )
            return True

        logger.info( 'Revoking certificate %s' % self.i.name )

        command = 'ca -config %s -name %s -batch -revoke %s -passin env:%s' % (PKI_OPENSSL_CONF, self.i.parent, self.crt, self.env_pw)
        self.exec_openssl(command.split(), env_vars={ self.env_pw: str(ppf) })

    def renew_certificate(self):
        '''Renew/Reissue a given certificate'''

        logger.info( 'Renewing certificate %s' % self.i.name )

        if os.path.exists(self.csr):
            self.sign_csr()
        else:
            raise Exception( "Failed to renew certificate %s! CSR is missing!" % self.i.name )

    @in_temp_dir
    def verify_ca_chain(self, chain):
        """Verify the the CA chain
        """
        trusted_chain = [crt for crt in chain if crt.trust]
        certs = "".join([crt.pem for crt in chain ])

        for c in trusted_chain:
            filepath = os.path.join(self.tmpdir, "%s.0" % c.certhash)
            w = open(filepath, 'w')
            w.write(c.pem)
            w.close()
        command = ['verify', '-CApath', self.tmpdir, ]

        result = self.exec_openssl(command, stdin=certs)

        if result == "stdin: OK\n":
            return True
        else:
            raise self.VerifyError(result)

    def get_hash_from_cert(self, cert):
        """Use openssl to get the hash value of a given certificate
        """
        command = 'x509 -hash -noout'
        output  = self.exec_openssl(command.split(), cert)

        return output.rstrip("\n")

    def get_subject_from_cert(self, cert):
        """Get the subject form a given CA certificate
        """

        command = ["x509", "-noout", "-subject"]
        output  = self.exec_openssl(command, stdin=cert.pem)
        return output.rstrip("\n").lstrip("subject= ")

    def get_revoke_status_from_cert(self, cert, crl):
        """Is the given certificate already revoked? True=yes, False=no

        Beware : that don't check cert <-> crl signature
        """
        serial = cert.serial

        command = 'crl -text -noout'
        output  = self.exec_openssl(command.split(), stdin=crl)

        serial_re = re.compile('^\s+Serial\sNumber\:\s+(\w+)')
        lines = output.split('\n')
        serial = serial.rjust(2,"0")

        for l in lines:
            if serial_re.match(l):
                if serial_re.match(l).group(1) == serial:
                    #logger.info( "The certificate is revoked" )
                    return True

        return False

    def generate_index(self, ca, issued):
        """Generate Index
        http://www.mail-archive.com/openssl-users@openssl.org/msg45982.html

        The columns are defined as 
        #define DB_type         0 /* Status of the certificate */
        #define DB_exp_date     1 /* Expiry date */
        #define DB_rev_date     2 /* Revocation date */
        #define DB_serial       3       /* Serial No., index - unique */
        #define DB_file         4      
        #define DB_name         5       /* DN, index - unique when active and not disabled */

        DB_type is defined as
        #define DB_TYPE_REV    'R' /* Revoked */
        #define DB_TYPE_EXP    'E' /* Expired */
        #define DB_TYPE_VAL    'V' /* Valid */

        /!\ Only revoked for now ...

        """
        raise NotImplementedError()
        from M2Crypto.ASN1 import ASN1_UTCTIME
        index = ""
        for cert in issued:
            if cert.revoked:
                subject = self.get_subject_from_cert(cert)
                asn1 = ASN1_UTCTIME()
                asn1.set_datetime(cert)
                #expiry = asn1.
                # TODO : How can we output datetimes in ASN1_UTCTIME Format ???


    @in_temp_dir
    def generate_crl(self, ca, cakey, crlnumber, crlchain=[], issued=[], passphrase=None):
        """Generate CRL: When a CA is modified
        """
        certdir = os.path.join(self.tmpdir, "certs")
        os.mkdir(certdir, 0700)
        privdir = os.path.join(self.tmpdir, "private")
        os.mkdir(privdir, 0700)
        crldir = os.path.join(self.tmpdir, "crl")
        os.mkdir(crldir, 0700)

        # Issued
        for c in issued:
            filepath = os.path.join(certdir, "%s.0" % c.certhash)
            w = open(filepath, 'w')
            w.write(c.pem)
            w.close()

        # CA cert
        filepath = os.path.join(self.tmpdir, "cacert.pem")
        w = open(filepath, 'w')
        w.write(ca.pem)
        w.close()

        # CA key
        filepath = os.path.join(privdir, "cakey.pem")
        w = open(filepath, 'w')
        w.write(cakey)
        w.close()

        # Config
        shutil.copy(PKI_OPENSSL_CONF, self.tmpdir)
        confpath = os.path.join(self.tmpdir, os.path.split(PKI_OPENSSL_CONF)[-1])

        # Generate Index file
        open(os.path.join(self.tmpdir, "index.txt"), "w").write(ca.index)
        # Generate Serial file
        serial = "%X" % ca.ca_serial
        serial = serial.rjust(2,"0")
        open(os.path.join(self.tmpdir, "serial"), "w").write(serial)
        # Generate crlnumber file
        crlnumber = "%X" % crlnumber
        crlnumber = crlnumber.rjust(2,"0")
        open(os.path.join(self.tmpdir, "crlnumber"), "w").write(crlnumber)
        #open(os.path.join(self.tmpdir, "crlnumber"), "w").write("")

        # crl
        crlpath = os.path.join(self.tmpdir, "%s.r0" % ca.certhash)
        if ca.crl:
            w = open(crlpath, 'w')
            w.write(ca.crl)
            w.close()

        logger.info( 'CRL generation for CA %s' % ca )
        command = ["ca", "-config", confpath, "-gencrl", "-crldays", "1", "-passin", "stdin", "-out", crlpath]
        result = self.exec_openssl(command, stdin=passphrase, cwd=self.tmpdir)
        crlpem = open(crlpath, 'r').read()
        return crlpem


    @in_temp_dir
    def revoke_cert(self, ca, cakey, crlnumber, crl, cert, issued=[], passphrase=None):
        """Generate CRL: When a CA is modified
        """
        certdir = os.path.join(self.tmpdir, "certs")
        os.mkdir(certdir, 0700)
        privdir = os.path.join(self.tmpdir, "private")
        os.mkdir(privdir, 0700)
        crldir = os.path.join(self.tmpdir, "crl")
        os.mkdir(crldir, 0700)

        # Issued
        for c in issued:
            filepath = os.path.join(certdir, "%s.0" % c.certhash)
            if c == cert:
                torevoke = filepath
            w = open(filepath, 'w')
            w.write(c.pem)
            w.close()

        # CA cert
        filepath = os.path.join(self.tmpdir, "cacert.pem")
        w = open(filepath, 'w')
        w.write(ca.pem)
        w.close()

        # CA key
        filepath = os.path.join(privdir, "cakey.pem")
        keyfile = filepath
        w = open(filepath, 'w')
        w.write(cakey)
        w.close()

        # Config
        shutil.copy(PKI_OPENSSL_CONF, self.tmpdir)
        confpath = os.path.join(self.tmpdir, os.path.split(PKI_OPENSSL_CONF)[-1])

        # Generate Index file
        open(os.path.join(self.tmpdir, "index.txt"), "w").write(ca.index)
        # Generate Serial file
        serial = "%X" % ca.ca_serial
        serial = serial.rjust(2,"0")
        open(os.path.join(self.tmpdir, "serial"), "w").write(serial)
        # Generate crlnumber file
        crlnumber = "%X" % crlnumber
        crlnumber = crlnumber.rjust(2,"0")
        open(os.path.join(self.tmpdir, "crlnumber"), "w").write(crlnumber)
        #open(os.path.join(self.tmpdir, "crlnumber"), "w").write("")

        # crl
        crlpath = os.path.join(self.tmpdir, "crl.pem")
        if crl:
            w = open(crlpath, 'w')
            w.write(crl)
            w.close()

        #shutil.copytree(self.tmpdir, "/tmp/pkitest")
        command = ["ca", "-config", confpath, "-batch", "-revoke", torevoke, "-passin", "stdin" ]
        result = self.exec_openssl(command, stdin=passphrase, cwd=self.tmpdir)
        command = ["ca", "-config", confpath, "-gencrl", "-crldays", "1", "-passin", "stdin", "-out", crlpath]
        result = self.exec_openssl(command, stdin=passphrase, cwd=self.tmpdir)
        crlpem = open(crlpath, 'r').read()
        index = open(os.path.join(self.tmpdir, "index.txt"), 'r').read()
        return crlpem, index

    @in_temp_dir
    def sign_pkcs7(self, cert, text, key, certs=[], passphrase=None):
        """Make pkcs7 with smime
        """
        textpath = os.path.join(self.tmpdir, "in.txt")
        f = open(textpath, 'w')
        f.write(text)
        f.close()

        certpath = os.path.join(self.tmpdir, "cert.pem")
        f = open(certpath, 'w')
        f.write(cert)
        f.close()

        keypath = os.path.join(self.tmpdir, "key.pem")
        f = open(keypath, 'w')
        f.write(key)
        f.close()

        #print [open(certpath, 'r').read()]

        # TODO : add extra certificates
        #certs = "".join([crt.pem for crt in chain ])
        #extrapath = os.path.join(self.tmpdir, "extra.pem")
        #f = open(extrapath, 'w')
        #f.write(certs)
        #f.close()

        outpath = os.path.join(self.tmpdir, "out.pkcs7")

        command = ['smime', '-sign', '-text',
                '-in', textpath,
                '-out', outpath,
                '-signer', certpath,
                '-inkey', keypath,
                '-passin', 'stdin',
        #        '-certfile', extrapath,
                  ]
        result = self.exec_openssl(command, stdin=certs)

        f = open(outpath, 'r')

        return f.read()

    @in_temp_dir
    def verify_pkcs7(self, cert, smime, certs=[]):
        """Verify pcks7 smime
        """
        textpath = os.path.join(self.tmpdir, "in.pkcs7")
        f = open(textpath, 'w')
        f.write(smime)
        f.close()

        certpath = os.path.join(self.tmpdir, "cert.pem")
        f = open(certpath, 'w')
        f.write(cert)
        f.close()

        #print [open(certpath, 'r').read()]

        # TODO : add extra certificates
        #certs = "".join([crt.pem for crt in chain ])
        #extrapath = os.path.join(self.tmpdir, "extra.pem")
        #f = open(extrapath, 'w')
        #f.write(certs)
        #f.close()

        outpath = os.path.join(self.tmpdir, "out.text")

        command = ['smime', '-verify',
                '-in', textpath,
                '-out', outpath,
                '-signer', certpath,
                '-noverify', certpath,
        #        '-certfile', extrapath,
                  ]
        result = self.exec_openssl(command, stdin=certs)

        f = open(outpath, 'r')
        data = f.read()
        header = "Content-Type: text/plain\r\n\r\n"
        return data[len(header):]



import os
from setuptools import setup



readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

setup(
        name     = 'django-signature',
        version  = '0.2',
        packages = ['signature'],

        requires = ['python (>= 2.4)', 'django (>= 1.1)', "M2Crypto (>= 0.18)"],

        description  = 'Django application to generate and sign Models.',
        long_description = readme,
        author       = 'Johan Charpentier',
        author_email = 'jcharpentier@bearstech.com',
        url          = 'http://github.com/bearstech/django-signature',
        download_url = 'http://bitbucket.org/bearstech/django-signature/downloads',
        license      = 'GPL v3',
        keywords     = 'django models crypto openssl pki',
        classifiers  = [
                    'Development Status :: 4 - Beta',
                    'Environment :: Web Environment',
                    'Framework :: Django',
                    'Intended Audience :: Developers',
                    'Programming Language :: Python',
                    'Topic :: Software Development :: Libraries :: Python Modules',
                ],
)




import datetime
from django import forms
from django.core.serializers.pyyaml import Serializer as DjangoYAMLEncoder
from django.core.serializers.pyyaml import DjangoSafeDumper
from django.core import serializers
from django.db import models
from django.forms import fields

from StringIO import StringIO

from django.db import models
from django.utils.encoding import smart_str, smart_unicode
from django.utils import datetime_safe

import yaml
from hashlib import sha512


# The code here is a copy of django/core/serializers source files
# With adjonctions for FileFields sha512

class SMIMESerializer(DjangoYAMLEncoder):
    """
    Serializer for signatures
    """
    def serialize(self, queryset, **options):
        """
        Serialize a queryset.
        """
        self.options = options

        self.stream = options.get("stream", StringIO())
        self.selected_fields = options.get("fields")
        self.use_natural_keys = options.get("use_natural_keys", False)

        self.start_serialization()
        for obj in queryset:
            self.start_object(obj)
            for field in obj._meta.local_fields:
                if field.serialize:
                    if field.rel is None:
                        if self.selected_fields is None or field.attname in self.selected_fields:
                            self.handle_field(obj, field)
                    else:
                        if self.selected_fields is None or field.attname[:-3] in self.selected_fields:
                            self.handle_fk_field(obj, field)
            for field in obj._meta.many_to_many:
                if field.serialize:
                    if self.selected_fields is None or field.attname in self.selected_fields:
                        self.handle_m2m_field(obj, field)
            self.end_object(obj)
        self.end_serialization()
        return self.getvalue()

    def handle_field(self, obj, field):
        # A nasty special case: base YAML doesn't support serialization of time
        # types (as opposed to dates or datetimes, which it does support). Since
        # we want to use the "safe" serializer for better interoperability, we
        # need to do something with those pesky times. Converting 'em to strings
        # isn't perfect, but it's better than a "!!python/time" type which would
        # halt deserialization under any other language.
        if isinstance(field, models.TimeField) and getattr(obj, field.name) is not None:
            self._current[field.name] = str(getattr(obj, field.name))
        elif isinstance(field, models.FileField) and getattr(obj, field.name) is not None:
            # Get a sha512 of the File
            f = getattr(obj, field.name).open()
            if not f:
                # TODO : Ugly no?
                f = open(getattr(obj, field.name).name, 'rb')
            f.seek(0)
            sha = sha512()
            data = f.read(512)
            while len(data):
                sha.update(data) 
                data = f.read(512)
            self._current[field.name] = str(sha.hexdigest())
        else:
            super(SMIMESerializer, self).handle_field(obj, field)

    def handle_fk_field(self, obj, field):
        related = getattr(obj, field.name)
        if related is not None:
            if self.use_natural_keys and hasattr(related, 'natural_key'):
                related = related.natural_key()
            else:
                if field.rel.field_name == related._meta.pk.name:
                    # Related to remote object via primary key
                    related = related._get_pk_val()
                else:
                    # Related to remote object via other field
                    related = smart_unicode(getattr(related, field.rel.field_name), strings_only=True)
        self._current[field.name] = related

    def end_serialization(self):
        self.options.pop('stream', None)
        self.options.pop('fields', None)
        self.options.pop('use_natural_keys', None)
        yaml.dump(self.objects, self.stream, Dumper=DjangoSafeDumper, **self.options)

def serialize(obj, use_natural_keys=False):
    """
    """
    data = [obj]
    serializer = SMIMESerializer()
    serialized = serializer.serialize(data, use_natural_keys=use_natural_keys)
    return serialized
